package jdbcFinal;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Interface for interacting with database via JDBC driver.
 *
 * @author Sukharev Dmitriy
 *
 */
public interface DatabaseManager {
    /**
     * Loads JDBC driver.
     * 
     * @throws ClassNotFoundException
     */
    public void loadDriver() throws ClassNotFoundException;

    /**
     * Connects to database.
     * 
     * @param database
     * @return
     * @throws SQLException
     */
    public Connection connectToDB(String database) throws SQLException;

    /**
     * Disconnects from database.
     * 
     * @param connection
     * @throws SQLException
     */
    public void disconnectFromDB(Connection connection)
            throws SQLException;
    
    /**
     * Executes query to database: check if table exists in database.
     * 
     * @param connection
     * @param tablename
     * @throws SQLException
     */
    public boolean isTableExists(Connection connection, String tablename)
            throws SQLException;

    /**
     * Executes query to database: create table.
     * 
     * @param connection
     * @param tablename
     * @throws SQLException
     */
    public void createTable(Connection connection, String tablename)
            throws SQLException;

    /**
     * Executes query to database: fetch rows from database.
     * 
     * @param connection
     * @param tablename
     * @return
     * @throws SQLException
     */
    public ResultSet fetch(Connection connection, String tablename)
            throws SQLException;

    /**
     * Prints result of fetching to console.
     * 
     * @param resultSet
     * @throws SQLException
     */
    public void printFetched(ResultSet resultSet) throws SQLException;

    /**
     * Executes query to database: insert row to database.
     * 
     * @param connection
     * @param tablename
     * @return
     * @throws SQLException 
     */
    public void insert(Connection connection, String tablename) throws SQLException;
    
    /**
     * Executes query to database: update rows of table.
     * 
     * @param connection
     * @param tablename
     * @return
     * @throws SQLException 
     */
    public void update(Connection connection, String tablename) throws SQLException;
    
    /**
     * Executes query to database: delete rows from table.
     * 
     * @param connection
     * @param tablename
     * @return
     * @throws SQLException 
     */
    public void delete(Connection connection, String tablename) throws SQLException;
}
