/**
 * 
 */
package jdbcFinal;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * Interacts with MySQL database via JDBC driver.
 *
 * @author Sukharev Dmitriy
 *
 */
public class MySqlManager implements DatabaseManager {
    private String user = "user";
    private String pw = "pw";

    
    /**
     * Loads mysql jdbc driver.
     * 
     * @throws ClassNotFoundException
     */
    public void loadDriver() throws ClassNotFoundException {
        Class.forName("com.mysql.jdbc.Driver");
    }

    /**
     * Connects to MySQL.
     * 
     * @param database
     * @return
     * @throws SQLException
     */
    public Connection connectToDB(String database) throws SQLException {
        String url = "jdbc:mysql://localhost/" + database
                + "?autoReconnect=true&useUnicode=true&characterEncoding=utf8";
        Connection connection = DriverManager.getConnection(url, user, pw);

        return connection;
    }

    /**
     * Disconnects from MySQL.
     * 
     * @param connection
     * @throws SQLException
     */
    public void disconnectFromDB(Connection connection)
            throws SQLException {
        connection.close();
    }
    
    /**
     * Executes query to MySQL: check if table exists in DB.
     * 
     * @param connection
     * @param tablename
     * @throws SQLException
     */
    public boolean isTableExists(Connection connection, String tablename)
            throws SQLException {
        Statement statement = connection.createStatement();
        ResultSet resultSet;
        resultSet = statement.executeQuery("SHOW TABLES LIKE \"" + tablename + "\"");
        if (!resultSet.next()) {
            return false;
        }
        return true;
    }

    /**
     * Executes query to MySQL: create table.
     * 
     * @param connection
     * @param tablename
     * @throws SQLException
     */
    public void createTable(Connection connection, String tablename)
            throws SQLException {
        Statement statement = connection.createStatement();
        statement.executeUpdate("CREATE TABLE `" + tablename
                + "` (" + " `id` int(11) NOT NULL auto_increment,"
                + " `title` varchar(100) NOT NULL,"
                + " `comments` varchar(500) default NULL,"
                + " `price` double default NULL," + " PRIMARY KEY (`id`)"
                + " ) DEFAULT CHARSET=utf8");
    }

    /**
     * Executes query to MySQL: fetch rows from DB.
     * 
     * @param connection
     * @param tablename
     * @return
     * @throws SQLException
     */
    public ResultSet fetch(Connection connection, String tablename)
            throws SQLException {
        ResultSet resultSet = null;
        Statement statement = connection.createStatement();
        resultSet = statement.executeQuery("SELECT * FROM `" + tablename + "`");
        return resultSet;
    }

    /**
     * Prints result of fetching to console.
     * 
     * @param resultSet
     * @throws SQLException
     */
    public void printFetched(ResultSet resultSet) throws SQLException {
        while (resultSet.next()) {
            String id = resultSet.getString("id");
            String title = resultSet.getString("title");
            double price = resultSet.getDouble("price");
            System.out.println(id + "\t" + title + "\t" + price);
        }
    }

    /**
     * Executes query to MySQL: insert row to DB.
     * 
     * @param connection
     * @param tablename
     * @return
     * @throws SQLException
     */
    public void insert(Connection connection, String tablename)
            throws SQLException {
        Statement statement = connection.createStatement();
        statement.executeUpdate("INSERT INTO `" + tablename
                + "` VALUES(null, \"title\", \"comment\", \"2\")");
    }
    
    /**
     * Executes query to MySQL: update rows of table.
     * 
     * @param connection
     * @param tablename
     * @return
     * @throws SQLException
     */
    public void update(Connection connection, String tablename)
            throws SQLException {
        Statement statement = connection.createStatement();
        statement.executeUpdate("UPDATE `" + tablename
                        + "` SET title='title', comments='comment',"
                        + " price='2.7' WHERE id='1'");
    }

    /**
     * Executes query to MySQL: delete rows from table.
     * 
     * @param connection
     * @param tablename
     * @return
     * @throws SQLException
     */
    public void delete(Connection connection, String tablename)
            throws SQLException {
        Statement statement = connection.createStatement();
        statement.executeUpdate("DELETE FROM `" + tablename + "` WHERE id='1'");
    }
}
