/**
 * 
 */
package jdbcFinal;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * Interacts with Oracle Database via JDBC driver.
 *
 * @author Sukharev Dmitriy
 *
 */
public class OracleDBManager implements DatabaseManager {
	private String user = "system";
	private String pw = "pw";

	/**
	 * Loads Oracle JDBC driver.
	 * 
	 * @throws ClassNotFoundException
	 */
	public void loadDriver() throws ClassNotFoundException {
		Class.forName("oracle.jdbc.driver.OracleDriver");
	}

	/**
	 * Connects to OracleDB.
	 * 
	 * @param database
	 * @return
	 * @throws SQLException
	 */
	public Connection connectToDB(String database) throws SQLException {
		String url = "jdbc:oracle:thin:@//localhost:1521/" + database;
		Connection connection = DriverManager.getConnection(url, user, pw);

		return connection;
	}

	/**
	 * Disconnects from OracleDB.
	 * 
	 * @param connection
	 * @throws SQLException
	 */
	public void disconnectFromDB(Connection connection)
			throws SQLException {
		connection.close();
	}

	/**
	 * Executes query to OracleDB: check if table exists in database.
	 * 
	 * @param connection
	 * @param tablename
	 * @throws SQLException
	 */
	public boolean isTableExists(Connection connection, String tablename)
			throws SQLException {
		Statement statement = connection.createStatement();
		ResultSet resultSet;
		resultSet = statement.executeQuery("SELECT * FROM cat");
		while (resultSet.next()) {
			String fetchedTable = resultSet.getString("TABLE_NAME");
			if (fetchedTable.toUpperCase().equals(tablename.toUpperCase())) {
				return true;
			}
		}

		return false;
	}

    /**
     * Executes query to OracleDB: create table. Last two queries are for
     * auto-increment of id-field.
     * 
     * @param connection
     * @param tablename
     * @throws SQLException
     */
    public void createTable(Connection connection, String tablename)
            throws SQLException {
        Statement statement = connection.createStatement();
        statement.executeUpdate("CREATE TABLE " + tablename + " ("
                + " id NUMBER(11) NOT NULL PRIMARY KEY,"
                + " title VARCHAR2(100) NOT NULL,"
                + " comments VARCHAR2(500) NULL," + " price FLOAT NULL" + " )");

        statement.executeUpdate("CREATE SEQUENCE " + tablename + "_ID_SEQ"
                + " START WITH 1" + " INCREMENT BY 1" + " NOMAXVALUE");

        statement.executeUpdate("CREATE OR REPLACE TRIGGER " + tablename
                + "_ID_TRIGGER" + " BEFORE INSERT ON " + tablename
                + " REFERENCING NEW AS NEW OLD AS OLD FOR EACH ROW" + " BEGIN"
                + " SELECT " + tablename
                + "_ID_SEQ.NEXTVAL INTO :NEW.ID FROM DUAL; " + " END;");
    }

    /**
     * Executes query to OracleDB: fetch rows from database.
     * 
     * @param connection
     * @param tablename
     * @return
     * @throws SQLException
     */
    public ResultSet fetch(Connection connection, String tablename)
            throws SQLException {
        ResultSet resultSet = null;
        Statement statement = connection.createStatement();
        resultSet = statement.executeQuery("SELECT * FROM " + tablename);
        return resultSet;
    }

    /**
     * Prints fetched data to console.
     * 
     * @param resultSet
     * @throws SQLException
     */
    public void printFetched(ResultSet resultSet) throws SQLException {
        while (resultSet.next()) {
            String id = resultSet.getString("id");
            String title = resultSet.getString("title");
            double price = resultSet.getDouble("price");
            System.out.println(id + "\t" + title + "\t" + price);
        }
    }

    /**
     * Executes query to OracleDB: insert row to database.
     * 
     * @param connection
     * @param tablename
     * @return
     * @throws SQLException
     */
    public void insert(Connection connection, String tablename)
            throws SQLException {
        Statement statement = connection.createStatement();
        statement.executeUpdate("INSERT INTO " + tablename
                + " (title, comments, price)"
                + " VALUES('title', 'comment', '2')");
    }
    
    /**
     * Executes query to OracleDB: update rows of table.
     * 
     * @param connection
     * @param tablename
     * @return
     * @throws SQLException
     */
    public void update(Connection connection, String tablename)
            throws SQLException {
        Statement statement = connection.createStatement();
        statement.executeUpdate("UPDATE " + tablename
                        + " SET title='title', comments='comment',"
                        + " price='2.7' WHERE id='3'");
    }

    /**
     * Executes query to OracleDB: delete rows from table.
     * 
     * @param connection
     * @param tablename
     * @return
     * @throws SQLException
     */
    public void delete(Connection connection, String tablename)
            throws SQLException {
        Statement statement = connection.createStatement();
        statement.executeUpdate("DELETE FROM " + tablename + " WHERE id='1'");
    }
}
